@@ -2,11 +2,12 @@ from dexy.constants import Constants
 from dexy.version import Version
 from ordereddict import OrderedDict
 import codecs
+import dexy.commands
+import dexy.helpers
 import dexy.introspect
-import json
-import glob
 import hashlib
 import inspect
+import json
 import logging
 import os
 import shutil
@@ -21,7 +22,7 @@ class Artifact(object):
     MAX_DATA_DICT_DECIMALS = 5
     MAX_DATA_DICT_LENGTH = 10 ** MAX_DATA_DICT_DECIMALS
     META_ATTRS = [
-        'additional',
+        'additional_inputs',
         'binary_input',
         'binary_output',
         'created_by',
@@ -30,7 +31,6 @@ class Artifact(object):
         'final',
         'hashfunction',
         'initial',
-        'is_last',
         'logstream',
         'key',
         'name',
@@ -41,23 +41,28 @@ class Artifact(object):
     ]
 
     BINARY_EXTENSIONS = [
-         '.docx',
+        '.amr',
+        '.mp3',
+        '.docx',
+        '.eot',
         '.epub',
         '.gif',
-        '.jpg',
-        '.png',
-        '.pdf',
-        '.zip',
-        '.tgz',
         '.gz',
-        '.eot',
-        '.ttf',
+        '.jpg',
+        '.kch',
         '.odt',
+        '.pdf',
+        '.png',
         '.rtf',
-        '.woff',
         '.sqlite',
         '.sqlite3',
-        '.swf'
+        '.swf',
+        '.tgz',
+        '.ttf',
+        '.wav',
+        '.woff',
+        '.xls',
+        '.zip'
     ]
 
     def __init__(self):
@@ -66,6 +71,7 @@ class Artifact(object):
 
         self._inputs = {}
         self.additional = None
+        self.additional_inputs = []
         self.args = {}
         self.args['globals'] = {}
         self.artifacts_dir = 'artifacts' # TODO don't hard code
@@ -99,6 +105,40 @@ class Artifact(object):
         self.start_time = None
         self.state = 'new'
         self.stdout = None
+        self.virtual_docs = None
+
+    def keys(self):
+        return self.data_dict.keys()
+
+    def may_have_kv_storage(self):
+        return self.binary_output and (self.ext in dexy.helpers.KeyValueData.EXTENSIONS)
+
+    def __getitem__(self, key):
+        if not hasattr(self, "_storage") and self.binary_output and (self.ext in dexy.helpers.KeyValueData.EXTENSIONS):
+            self.setup_kv_storage()
+
+        if hasattr(self, "_storage"):
+            if self._storage.mode == "write":
+                # Change from write mode to read mode...
+                self.setup_kv_storage()
+            return self._storage.retrieve(key)
+        elif self.data_dict.has_key(key):
+            return self.data_dict[key]
+        elif hasattr(self, key):
+            return getattr(self, key)
+        elif self.ext in dexy.helpers.KeyValueData.EXTENSIONS:
+            self.setup_kv_storage()
+            return self._storage.retrieve(key)
+        else:
+            raise dexy.commands.UserFeedback("Can't find key '%s' in %s" % (key, self.key))
+
+    def __unicode__(self):
+        """
+        When d[key] is used without attributes being accessed, need to return artifact output text.
+
+        Jinja calls the __unicode__ method so we override that.
+        """
+        return self.output_text()
 
     def is_complete(self):
         return str(self.state) == 'complete'
@@ -144,12 +184,32 @@ class Artifact(object):
     def is_abstract(self):
         return not hasattr(self, 'save_meta')
 
+    def filter_args(self):
+        """
+        Returns args specified in the .dexy file for this filter alias.
+        """
+        args = {}
+        for a in self.filter_class.ALIASES:
+            if self.args.has_key(a):
+                try:
+                    args.update(self.args[a])
+                except ValueError as e:
+                    if "dictionary update sequence element" in e.message:
+                        raise dexy.commands.UserFeedback("You need to supply a dict to argument '%s', rather than the single value '%s'" % (a, self.args[a]))
+                    else:
+                        print self.args[a]
+                        raise e
+        return args
+
     def setup_initial(self):
         """
         Set up an initial artifact (the first artifact in a document's filter chain).
         """
-        self._inputs = {}
-        self.binary_input = (self.doc.ext in self.BINARY_EXTENSIONS)
+        if self.args.has_key('binary'):
+            self.binary_input = self.args['binary']
+        else:
+            self.binary_input = (self.doc.ext in self.BINARY_EXTENSIONS)
+
         self.binary_output = self.binary_input
         self.ext = self.doc.ext
         self.initial = True
@@ -216,9 +276,7 @@ class Artifact(object):
         # Look for additional inputs in previous artifacts or previous
         # artifacts' inputs.
         for k, a in previous_artifact.inputs().iteritems():
-
             if a.additional and not k in self._inputs:
-                self.log.debug("(%s) Adding additional artifact %s from %s" % (self.key, k, a.key))
                 self.add_input(k, a)
             elif not k in self._inputs and not a.virtual:
                 # We should have all other inputs already. Validate this.
@@ -226,7 +284,6 @@ class Artifact(object):
 
             for kk, aa in a.inputs().iteritems():
                 if aa.additional and not kk in self._inputs:
-                    self.log.debug("(%s) Adding additional artifact %s from %s" % (self.key, kk, k))
                     self.add_input(kk, aa)
 
         self.binary_input = previous_artifact.binary_output
@@ -249,8 +306,8 @@ class Artifact(object):
         else:
             next_inputs = None
 
-        if self.args.has_key('ext'):
-            ext = self.args['ext']
+        if self.filter_args().has_key('ext'):
+            ext = self.filter_args()['ext']
             if not ext.startswith("."):
                 ext = ".%s" % ext
             self.ext = ext
@@ -275,6 +332,7 @@ class Artifact(object):
         artifact = klass()
         artifact.key = artifact_key
         artifact.filter_class = filter_class
+        artifact.is_last = (artifact.key == doc.key())
 
         # Add references for convenience
         artifact.artifacts_dir = doc.artifacts_dir
@@ -286,7 +344,6 @@ class Artifact(object):
 
         # These attributes are the same for all artifacts pertaining to a document
         artifact.args = doc.args
-        artifact.log.debug("Setting artifact args for %s in %s to %s" % (artifact.key, doc.key(), doc.args))
         artifact.batch_id = doc.batch_id
         artifact.document_key = doc.key()
         artifact.name = doc.name
@@ -301,13 +358,19 @@ class Artifact(object):
 
         # Set inputs from original document inputs.
         artifact._inputs.update(artifact.doc.input_artifacts())
+        if len(artifact.doc.input_artifacts().keys()) > 10:
+            doc.log.debug("Setting inputs to include %s document inputs" % len(artifact.doc.input_artifacts()))
+        elif len(artifact.doc.input_artifacts().keys()) > 0:
+            doc.log.debug("Setting inputs to include inputs: %s" % ",".join(artifact.doc.input_artifacts().keys()))
 
         for k, a in artifact.doc.input_artifacts().iteritems():
             if a.additional and not k in artifact._inputs:
+                doc.log.debug("Adding additional input %s" % k)
                 artifact.add_input(k, a)
 
             for kk, aa in a.inputs().iteritems():
                 if aa.additional and not kk in artifact._inputs:
+                    doc.log.debug("Adding additional input %s" % kk)
                     artifact.add_input(kk, aa)
 
         if previous_artifact:
@@ -341,19 +404,34 @@ class Artifact(object):
 
             try:
                 filter_instance.process()
-            except Exception as e:
-                # TODO Clean this up, should all go to stderr probably.
-                print "Error occurred while running", self.key
-                x, y, tb = sys.exc_info()
-                print "Original traceback:"
-                traceback.print_tb(tb, sys.stdout)
-                pattern = os.path.join(self.artifacts_dir, self.hashstring)
-                files_matching = glob.glob(pattern)
-                if len(files_matching) > 0:
-                    print "Here are working files which might have clues about this error:"
-                    for f in files_matching:
-                        print f
+            except dexy.commands.UserFeedback as e:
+                messages = []
+                err_msg_args = (self.doc.key(), self.filter_alias, self.doc.step, len(self.doc.filters))
+                messages.append("ERROR in %s (in filter '%s' - step %s of %s)" % err_msg_args)
+                messages.append(e.message)
+
+                for message in messages:
+                    self.log.debug(message)
+
+                messages.append("This exception information has been written to logs/dexy.log")
+                messages.append("There may be more information in logs/dexy.log")
+                if self.log.getEffectiveLevel() > logging.DEBUG:
+                    messages.append("If you can't find clues in the log, try running again with -loglevel DEBUG")
+
+                raise dexy.commands.UserFeedback("\n".join(messages))
+
+            except dexy.commands.InternalDexyProblem as e:
+                err_msg_args = (self.doc.key(), self.filter_alias, self.doc.step, len(self.doc.filters))
+                sys.stderr.write("ERROR in %s (in filter '%s' - step %s of %s)\n" % err_msg_args)
                 raise e
+            except Exception as e:
+                traceback.print_tb(sys.exc_info()[2])
+                err_msg_args = (self.doc.key(), self.filter_alias, self.doc.step, len(self.doc.filters))
+                sys.stderr.write("ERROR in %s (in filter '%s' - step %s of %s)\n" % err_msg_args)
+                if e.message:
+                    raise dexy.commands.InternalDexyProblem("error class: %s\nerror message: %s" % (e.__class__.__name__, e.message))
+                else:
+                    raise dexy.commands.InternalDexyProblem("error class: %s" % e.__class__.__name__)
 
             if self.data_dict and len(self.data_dict) > 0:
                 pass
@@ -375,10 +453,7 @@ class Artifact(object):
 
             # make sure additional artifacts are added to db
             for a in self.inputs().values():
-                if a.created_by == self.key:
-                    if not a.additional:
-                        raise Exception("created_by should only apply to additional artifacts")
-                    # TODO Should this be done in Artifact.retrieve?
+                if a.additional and not a.key in self.db.extra_keys:
                     a.batch_id = self.batch_id
                     self.db.append_artifact(a)
 
@@ -386,11 +461,10 @@ class Artifact(object):
         self.db.update_artifact(self)
 
     def add_additional_artifact(self, key_with_ext, ext=None):
-        """create an 'additional' artifact with random hashstring"""
         if not ext:
             ext = os.path.splitext(key_with_ext)[1]
         new_artifact = self.__class__()
-        new_artifact.key = key_with_ext
+        new_artifact.key = os.path.splitext(key_with_ext)[0]
         if ext.startswith("."):
             new_artifact.ext = ext
         else:
@@ -401,12 +475,12 @@ class Artifact(object):
         new_artifact.set_binary_from_ext()
         new_artifact.artifacts_dir = self.artifacts_dir
         new_artifact.inode = self.hashstring
-        new_artifact.created_by = self.document_key
+        new_artifact.created_by = self.key
         new_artifact.virtual = True
-        # TODO filter class source?
+        new_artifact.name = key_with_ext.split("|")[0]
 
         # TODO this is duplicated in setup_from_previous_artifact, should reorganize
-        for at in ['batch_id', 'document_key', 'mtime', 'ctime']:
+        for at in ['batch_id', 'document_key', 'mtime', 'ctime', 'virtual_docs']:
                 val = getattr(self, at)
                 setattr(new_artifact, at, val)
 
@@ -418,6 +492,7 @@ class Artifact(object):
 
     def add_input(self, key, artifact):
         self._inputs[key] = artifact
+        self.additional_inputs.append(artifact.hashstring)
 
     def inputs(self):
         return self._inputs
@@ -547,7 +622,6 @@ class Artifact(object):
         except IOError as e:
             self.save_meta()
 
-
     def convert_if_not_unicode(self, s):
         if type(s) == unicode:
             return s
@@ -568,6 +642,12 @@ class Artifact(object):
     def output_text(self):
         return u"".join([self.convert_if_not_unicode(v) for k, v in self.data_dict.items()])
 
+    def read_binary_output(self):
+        self.binary_output = True
+        self.load_output()
+        self.binary_output = False
+        return self.binary_data
+
     def output(self):
         if not self.is_complete():
             raise Exception("can't call output unless complete!")
@@ -637,13 +717,14 @@ class Artifact(object):
                     input_artifact.write_to_file(filename)
                     self.log.debug("Populating temp dir for %s with %s" % (self.key, filename))
                 else:
-                    self.log.warn("Skipping file %s for temp dir for %s, file does not exist (yet)" % (filename, self.key))
+                    self.log.warn("Not populating temp dir for %s with file %s, file does not exist (yet)" % (self.key, filename))
 
             # write the workfile to this directory under its canonical name
             previous = self.previous_artifact_filepath
             workfile = os.path.join(tempdir, self.previous_canonical_filename)
             if not os.path.exists(os.path.dirname(workfile)):
                 os.makedirs(os.path.dirname(workfile))
+            self.log.debug("Copying %s to %s" % (previous, workfile))
             shutil.copyfile(previous, workfile)
 
     def alias(self):
@@ -682,6 +763,10 @@ class Artifact(object):
     def websafe_key(self):
         return self.long_canonical_filename().replace("/", "--")
 
+    def web_safe_document_key(self):
+        # duplicate, remove this alias
+        return self.websafe_key()
+
     def filename(self):
         """
         The filename where artifact content is stored, based on the hashstring.
@@ -722,9 +807,8 @@ class Artifact(object):
     def unique_key(self):
         return "%s:%s:%s" % (self.batch_id, self.document_key, self.key)
 
-    def web_safe_document_key(self):
-        # TODO this might not be unique
-        return self.document_key.replace("/","-").replace("|", "-")
+    def websafe_unique_key(self):
+        return self.unique_key().replace("/", "--")
 
     def url(self):
         # TODO test for final
@@ -756,12 +840,26 @@ class Artifact(object):
         # TODO test for final
         return """<img src="/%s" alt="Image generated by dexy %s" />""" % (self.canonical_filename(), self.key)
 
+    def relpath(self, artifact_key):
+        """
+        Returns relative path from self to other artifact key, e.g. for linking to CSS relatively
+        """
+        artifact = self.inputs()[artifact_key]
+        return os.path.join(self.relative_path_to_input(artifact), artifact.canonical_basename())
+
     def has_sections(self):
         return (self.data_dict.keys() != ['1'])
 
     def relative_path_to_input(self, input_artifact):
         my_dir = os.path.dirname(self.name)
         input_dir = os.path.dirname(input_artifact.name)
+        self.log.debug("Calculating relative path between %s and %s" % (self.name, input_artifact.name))
+
+        if not my_dir:
+            my_dir = "."
+        if not input_dir:
+            input_dir = "."
+
         if my_dir == input_dir:
             relpath = ""
         else:
@@ -792,3 +890,29 @@ class Artifact(object):
             fmt = "%%0%sd:%%s" % self.MAX_DATA_DICT_DECIMALS
             data_dict[fmt % (i, k)] = v
         return data_dict
+
+    def storage(self, reset=False):
+        if not hasattr(self, "_storage") or reset:
+            # Assume we want KV storage
+            self.setup_kv_storage()
+        return self._storage
+
+    def key_prefixes(self):
+        return sorted(set(":".join(k.split(":")[:-1]) for k in self.storage().keys()))
+
+    def kv_storage(self):
+        return self.storage()
+
+    def row_storage(self):
+        if not hasattr(self, "_storage"):
+            self.setup_row()
+        return self._storage
+
+    def setup_kv_storage(self):
+        try:
+            self._storage = dexy.helpers.KeyValueData(self.filepath())
+        except ValueError as e:
+            raise dexy.commands.UserFeedback("Can't get key-value data from %s for %s: %s" % self.filepath(), self.key, e.message)
+
+    def setup_row_storage(self):
+        self._storage = dexy.helpers.RowData(self.filepath())
