@@ -1,19 +1,39 @@
 from dexy.constants import Constants
-from dexy.controller import Controller
-from dexy.utils import get_log
 from dexy.version import Version
 from modargs import args
 from pygments import highlight
-from pygments.lexers.agile import PythonLexer
 from pygments.formatters import TerminalFormatter
+from pygments.lexers.agile import PythonLexer
 import cProfile
 import copy
 import datetime
+import dexy.controller
 import dexy.introspect
+import dexy.utils
 import inspect
+import json
 import os
+import platform
 import shutil
 import sys
+import warnings
+
+import web
+
+class InternalDexyProblem(Exception):
+    def __init__(self, message):
+        self.message = """\nOops! You may have found a bug in Dexy.
+        The developer would really appreciate if you copy and paste this entire message
+        and the Traceback above it into a bug report at http://dexy.tenderapp.com.
+        Your version of Dexy is %s
+        Your platform is %s\n""" % (Version.VERSION, platform.system())
+        self.message += message
+
+    def __str__(self):
+        return self.message
+
+class UserFeedback(Exception):
+    pass
 
 MOD = sys.modules[__name__]
 PROG = 'dexy'
@@ -30,7 +50,14 @@ NODOC_FILTERS = [
     ]
 
 def run():
-    args.parse_and_run_command(sys.argv[1:], MOD, default_command=Constants.DEFAULT_COMMAND)
+    warnings.filterwarnings("ignore",category=DeprecationWarning)
+    try:
+        args.parse_and_run_command(sys.argv[1:], MOD, default_command=Constants.DEFAULT_COMMAND)
+    except UserFeedback as e:
+        sys.stderr.write(e.message)
+        if not e.message.endswith("\n"):
+            sys.stderr.write("\n")
+        sys.exit(1)
 
 def dexy_command(
         allreports=False, # whether to run all available reports
@@ -46,7 +73,8 @@ def dexy_command(
         exclude="", # directories to exclude from dexy processing
         filters=False, # DEPRECATED just to catch people who use the old dexy --filters syntax
         globals="", # global values to make available within dexy documents, should be KEY=VALUE pairs separated by spaces
-        help=False, # DEPRECATED just to catch people who use the old dexy --help syntax
+        help=False, # for people who type -help out of habit
+        h=False, # for people who type -h out of habit
         hashfunction='md5', # What hash function to use, set to crc32 or adler32 for more speed, less reliability
         ignore=False, # whether to ignore nonzero exit status or raise an error - may not be supported by all filters
         inputs=False, # whether to log information about inputs for debugging
@@ -61,6 +89,7 @@ def dexy_command(
         reset=False, # whether to purge existing artifacts and logs before running Dexy
         run="", # specific document to run. if specified, this document + its dependencies will be all that is run
         setup=False, # DEPRECATED just to catch people who use the old dexy --setup syntax
+        silent=False, # Whether to not print any output when running dexy
         strictinherit=False, # set to true if you want 'allinputs' to only reference items in same dir or a subdir
         uselocals=True, # use cached local copies of remote URLs, faster but might not be up to date, 304 from server will override this setting
         version=False # DEPRECATED just to catch people who use the old dexy --version syntax
@@ -96,14 +125,12 @@ def dexy_command(
     If you run into trouble, visit http://dexy.it/help
     """
     # catch deprecated arguments
-    if help or version or reporters or filters:
-        print "the command syntax has changed, please type 'dexy help' for help"
-        sys.exit(1)
+    if h or help or version or reporters or filters:
+        raise UserFeedback("the command syntax has changed, please type 'dexy help' for help")
 
     ### @export "dexy-command-check-setup"
     if not check_setup(logsdir=logsdir, artifactsdir=artifactsdir):
-        print "Please run '%s setup' first to create the directories dexy needs to work with" % PROG
-        sys.exit(1)
+        raise UserFeedback("Please run '%s setup' first to create the directories dexy needs to work with" % PROG)
 
     ### @export "dexy-command-process-args"
     if reset:
@@ -111,14 +138,14 @@ def dexy_command(
 
     if output:
         if not reports == Constants.DEFAULT_REPORTS:
-            raise Exception("if you pass --output you can't also modify reports! pick 1!")
+            raise UserFeedback("if you pass --output you can't also modify reports! pick 1!")
         if allreports:
-            raise Exception("if you pass --output you can't also pass --allreports!")
+            raise UserFeedback("if you pass --output you can't also pass --allreports!")
         reports = "Output"
 
     if allreports:
         if not reports == Constants.DEFAULT_REPORTS:
-            raise Exception("if you pass --allreports you can't also specify --reports")
+            raise UserFeedback("if you pass --allreports you can't also specify --reports")
 
     controller = run_dexy(locals())
     if not dryrun:
@@ -144,7 +171,7 @@ def run_dexy(args):
     # validate args and do any conversions required
     args['globals'] = dict([g.split("=") for g in args['globals'].split()])
     args['exclude'] = [x.strip("/") for x in args['exclude'].split()]
-    controller = Controller(args)
+    controller = dexy.controller.Controller(args)
     controller.run()
     return controller
 
@@ -182,7 +209,13 @@ def profile_command(
 def check_setup(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT_ADIR):
     return os.path.exists(logsdir) and os.path.exists(artifactsdir)
 
-def setup_command(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT_ADIR, logfile=Constants.DEFAULT_LFILE, showhelp=True):
+def setup_command(
+        logsdir=Constants.DEFAULT_LDIR,
+        artifactsdir=Constants.DEFAULT_ADIR,
+        logfile=Constants.DEFAULT_LFILE,
+        showhelp=True,
+        scm="" # set to git or hg to automatically generate ignore files
+        ):
     """
     Creates directories to hold artifacts and logs. Dexy needs these
     directories and they aren't created automatically as a precaution against
@@ -196,9 +229,21 @@ def setup_command(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT
         if not os.path.exists(artifactsdir):
             os.mkdir(artifactsdir)
 
+        if not scm or len(scm) == 0:
+            pass
+        elif scm == 'git':
+            if not os.path.exists(".gitignore"):
+                with open(".gitignore", "w") as f:
+                    f.write("artifacts/\nlogs/\noutput/\noutput-long/\n*.sw*\n*.pyc\n")
+        elif scm == 'hg':
+            if not os.path.exists(".hgignore"):
+                with open(".hgignore", "w") as f:
+                    f.write("artifacts/\nlogs/\noutput/\noutput-long/\n")
+        else:
+            sys.stderr.write("Sorry, %s is not a supported scm for automatically creating an ignore file." % scm)
+
         if showhelp:
-            print
-            print "Ok, we've created directories called %s and %s." % (logsdir, artifactsdir)
+            print "Ok, we've created directories called %s and %s and a .%signore" % (logsdir, artifactsdir, scm)
             if os.path.exists(Constants.DEFAULT_CONFIG):
                 print "You are now ready to run dexy!  If you have problems,"
                 print "please check the log file at %s/%s for clues." % (logsdir, logfile)
@@ -208,7 +253,6 @@ def setup_command(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT
                 print "check out the tutorials dexy.it/docs/tutorials if you aren't sure how."
                 print "You can type '%s help -on %s' (without quotes) for help running dexy" % (PROG, Constants.DEFAULT_COMMAND)
                 print "or visit dexy.it/help for more resources"
-            print
 
 def cleanup_command(logsdir=Constants.DEFAULT_LDIR, artifactsdir=Constants.DEFAULT_ADIR):
     """
@@ -330,10 +374,10 @@ def filters_command(
         print "looking up filter information..."
     else:
         if showall:
-            raise Exception("can't specify an alias if showall is True")
+            raise UserFeedback("can't specify an alias if showall is True, exiting...")
 
     if check_setup():
-        log = get_log()
+        log = dexy.utils.get_log()
     else:
         log = Constants.NULL_LOGGER
 
@@ -359,7 +403,7 @@ def filters_text(
         text.append("")
         text.append("Aliases: %s" % ", ".join(klass.ALIASES))
         text.append("")
-        text.append(trim_docstring(klass.__doc__))
+        text.append(inspect.getdoc(klass))
         text.append("")
         text.append("http://dexy.it/docs/filters/%s" % alias)
         if source:
@@ -404,7 +448,7 @@ def filters_text(
 
             if not skip:
                 name_and_aliases = "%s (%s) " % (klass.__name__, ", ".join(klass.ALIASES))
-                docstring = trim_docstring(klass.__doc__)
+                docstring = inspect.getdoc(klass) or ""
                 if "\n" in docstring:
                     docstring = docstring.splitlines()[0]
                 filter_help = name_and_aliases + docstring
@@ -430,7 +474,7 @@ def reporters_text(log):
         text.append("\n" + r.__name__.replace("Reporter", ""))
 
         if r.__doc__:
-            text.append(S + r.__doc__)
+            text.append(S + inspect.getdoc(r))
         else:
             text.append(S + "no documentation available")
 
@@ -481,7 +525,7 @@ def reports_command(
     for r in reports:
         reporter_class_name = r.endswith("Reporter") and r or ("%sReporter" % r)
         if not reporters.has_key(reporter_class_name):
-            raise Exception("No reporter class named %s available. Valid reporter classes are: %s" % (r, ", ".join(reporters.keys())))
+            raise UserFeedback("No reporter class named %s available. Valid reporter classes are: %s" % (r, ", ".join(reporters.keys())))
         report_class = reporters[reporter_class_name]
 
         print "running", r
@@ -503,7 +547,7 @@ def fcmds_command(alias=None):
     These commands can then be run using the fcmd command.
     """
     if check_setup():
-        log = get_log()
+        log = dexy.utils.get_log()
     else:
         log = Constants.NULL_LOGGER
 
@@ -533,7 +577,7 @@ def fcmd_command(
     Run a command defined in a dexy filter.
     """
     if check_setup():
-        log = get_log()
+        log = dexy.utils.get_log()
     else:
         log = Constants.NULL_LOGGER
 
@@ -543,42 +587,122 @@ def fcmd_command(
     cmd_name = "docmd_%s" % cmd
 
     if not filter_class.__dict__.has_key(cmd_name):
-        raise Exception("%s is not a valid command. There is no method %s defined in %s" % (cmd, cmd_name, filter_class.__name__))
+        raise UserFeedback("%s is not a valid command. There is no method %s defined in %s" % (cmd, cmd_name, filter_class.__name__))
     else:
         class_method = filter_class.__dict__[cmd_name]
         if type(class_method) == classmethod:
             if help:
-                print class_method.__func__.__doc__
+                print inspect.getdoc(class_method.__func__)
             else:
-                class_method.__func__(filter_class, **kwargs)
+                try:
+                    class_method.__func__(filter_class, **kwargs)
+                except TypeError as e:
+                    print e.message
+                    print inspect.getargspec(class_method.__func__)
+                    print inspect.getdoc(class_method.__func__)
+                    raise e
+
         else:
-            raise Exception("expected %s to be a classmethod of %s" % (cmd_name, filter_class.__name__))
+            raise InternalDexyProblem("expected %s to be a classmethod of %s" % (cmd_name, filter_class.__name__))
 
 def it_command(**kwargs):
     dexy_command(kwargs)
 
-# From http://www.python.org/dev/peps/pep-0257/#handling-docstring-indentation
-def trim_docstring(docstring):
-    if not docstring:
-        return ''
-    # Convert tabs to spaces (following the normal Python rules)
-    # and split into a list of lines:
-    lines = docstring.expandtabs().splitlines()
-    # Determine minimum indentation (first line doesn't count):
-    indent = sys.maxint
-    for line in lines[1:]:
-        stripped = line.lstrip()
-        if stripped:
-            indent = min(indent, len(line) - len(stripped))
-    # Remove indentation (first line is special):
-    trimmed = [lines[0].strip()]
-    if indent < sys.maxint:
-        for line in lines[1:]:
-            trimmed.append(line[indent:].rstrip())
-    # Strip off trailing and leading blank lines:
-    while trimmed and not trimmed[-1]:
-        trimmed.pop()
-    while trimmed and not trimmed[0]:
-        trimmed.pop(0)
-    # Return a single string:
-    return '\n'.join(trimmed)
+def grep_command(
+        expr=None, # The expression to search for
+        keyexpr="", # Only search for keys matching this expression, implies keys=True
+        keys=False, # if True, try to list the keys in any found files
+        recurse=False, # if True, recurse into keys to look for sub keys (implies keys=True)
+        artifactclass=Constants.DEFAULT_ACLASS, # name of class to use for artifacts
+        dbclass=Constants.DEFAULT_DBCLASS, # name of database class to use
+        dbfile=Constants.DEFAULT_DBFILE, # name of the database file (it lives in the logs dir)
+        logsdir=Constants.DEFAULT_LDIR # location of directory in which to store logs
+        ):
+    """
+    Search for a Dexy document in the database matching the expression.
+
+    For sqlite the expression will be wrapped in % for you.
+    """
+    db = dexy.utils.get_db(dbclass, dbfile=dbfile, logsdir=logsdir)
+    for row in db.query_like("%%%s%%" % expr):
+        print row['key']
+        if keys or len(keyexpr) > 0 or recurse:
+            artifact_classes = dexy.introspect.artifact_classes()
+            artifact_class = artifact_classes[artifactclass]
+            artifact = artifact_class.retrieve(row['hashstring'])
+            if artifact.ext in [".json", ".kch", ".sqlite3"]:
+                if len(keyexpr) > 0:
+                    rows = artifact.kv_storage().query("%%%s%%" % keyexpr)
+                else:
+                    rows = artifact.kv_storage().keys()
+
+                if rows:
+                    print "  key-value store keys:"
+                for k in rows:
+                    print "    %s" % k
+                    if recurse:
+                        v = artifact.retrieve_from_kv_storage(k)
+                        try:
+                            if not hasattr(v, "keys"):
+                                v = json.loads(v)
+                            if hasattr(v, "keys"):
+                                for kk in v.keys():
+                                    print "      %s" % kk
+                        except Exception as e:
+                            pass
+
+            if len(artifact.data_dict.keys()) > 1:
+                print "  data dict keys:"
+            for k in artifact.data_dict.keys():
+                if not k == '1':
+                    print "    %s" % k
+
+
+class CustomStaticApp(web.httpserver.StaticApp):
+    def translate_path(self, path):
+        return os.path.join(os.path.dirname(os.path.abspath(__file__)), "viewer", path.lstrip("/"))
+
+import posixpath
+import urllib
+class StaticMiddleware:
+    """WSGI middleware for serving static files."""
+    def __init__(self, app, prefix='/static/'):
+        self.app = app
+        self.prefix = prefix
+
+    def __call__(self, environ, start_response):
+        path = environ.get('PATH_INFO', '')
+        path = self.normpath(path)
+
+        if path.startswith(self.prefix):
+            return CustomStaticApp(environ, start_response)
+        else:
+            return self.app(environ, start_response)
+
+    def normpath(self, path):
+        path2 = posixpath.normpath(urllib.unquote(path))
+        if path.endswith("/"):
+            path2 += "/"
+        return path2
+
+def viewer_command(
+        port=8090 # Port on which to run the viewer
+        ):
+    """
+    Starts a web.py application which lets you preview the snippets generated by your previous dexy run.
+    """
+    import dexy.viewer.app as viewer
+    func = viewer.app.wsgifunc()
+    server_address =("0.0.0.0", port)
+
+    func = StaticMiddleware(func)
+    func = web.httpserver.LogMiddleware(func)
+
+    server = web.httpserver.WSGIServer(server_address, func)
+
+    print "http://%s:%d/" % server_address
+    print "Type ctrl+c to stop the server."
+    try:
+         server.start()
+    except KeyboardInterrupt:
+         server.stop()
