@@ -1,13 +1,13 @@
 from dexy.constants import Constants
 from dexy.topsort import CycleError
 from dexy.topsort import topsort
-from dexy.utils import get_log
-from dexy.utils import save_batch_info
 from ordereddict import OrderedDict
 import copy
 import dexy
+import dexy.commands
 import dexy.document
 import dexy.introspect
+import dexy.utils
 import fnmatch
 import glob
 import json
@@ -30,7 +30,7 @@ class Controller(object):
 
         # Set up logging
         if args.has_key("logsdir") and args.has_key("logfile"):
-            self.log = get_log("dexy.controller", args['logsdir'], args['logfile'], args['loglevel'])
+            self.log = dexy.utils.get_log("dexy.controller", args['logsdir'], args['logfile'], args['loglevel'])
         else:
             self.log = Constants.NULL_LOGGER
 
@@ -51,7 +51,7 @@ class Controller(object):
             if self.artifact_classes.has_key(args['artifactclass']):
                 self.artifact_class = self.artifact_classes[args['artifactclass']]
             else:
-                raise Exception("Artifact class name %s not found in %s" % (args['artifactclass'], ",".join(self.artifact_classes.keys())))
+                raise dexy.commands.UserFeedback("Artifact class name %s not found in %s" % (args['artifactclass'], ",".join(self.artifact_classes.keys())))
 
 
     def run(self):
@@ -81,9 +81,14 @@ class Controller(object):
         # set the list of documents which are virtual
         self.virtual_docs = [d for d in self.docs if d.virtual]
 
-        if not self.args['dryrun']:
-            [doc.setup() for doc in self.docs]
-            self.docs = [doc.run() for doc in self.docs]
+        try:
+            if not self.args['dryrun']:
+                [doc.setup() for doc in self.docs]
+                self.docs = [doc.run() for doc in self.docs]
+        except dexy.commands.UserFeedback as e:
+            self.persist()
+            raise e
+
         self.timing.append(("run-docs", time.time() - start))
 
         self.batch_finish_time = time.time()
@@ -100,7 +105,7 @@ class Controller(object):
         JSON file (for use by reporters or for debugging).
         """
         self.db.persist()
-        save_batch_info(self.batch_id, self.batch_info(), self.args['logsdir'])
+        dexy.utils.save_batch_info(self.batch_id, self.batch_info(), self.args['logsdir'])
 
     def batch_info(self):
         """
@@ -123,6 +128,8 @@ class Controller(object):
         parent directory (up as far as the dexy project root) for config files
         and combining them, such that subdirectories override parents.
         """
+        self.log.debug("Determining configuration applicable in %s" % path)
+
         global_args = {}
         config_dict = {}
         variables = {}
@@ -148,7 +155,7 @@ class Controller(object):
                         json_dict = json.load(cf)
                     except ValueError as e:
                         msg = "Your config file %s has invalid JSON\n%s" % (f, e.message)
-                        raise Exception(msg)
+                        raise dexy.commands.UserFeedback(msg)
 
                 if json_dict.has_key("$reset"):
                     # Reset the config, i.e. ignore everything from parent
@@ -237,7 +244,6 @@ class Controller(object):
 
         # Define the parse_doc nested function which we will call recursively.
         def parse_doc(path, input_directive, args = {}):
-            self.log.debug("Parsing doc path: '%s' input_dir: '%s' args : '%s'" % (path, input_directive, args))
             # If a specification is nested in a dependency, then input_directive
             # may be a dict. If so, split it into parts before continuing.
             try:
@@ -249,7 +255,7 @@ class Controller(object):
 
             tokens = input_directive.split("|")
             if "/" in tokens[0]:
-                raise Exception("paths not allowed in tokens: %s" % tokens[0])
+                raise dexy.commands.UserFeedback("paths not allowed in tokens: %s" % tokens[0])
             if path == '.':
                 glob_string = tokens[0]
             else:
@@ -261,12 +267,11 @@ class Controller(object):
             # virtual document
             if re.search("@", glob_string):
                 virtual = True
-
-                dangerous = not args.has_key('contents')
+                dangerous = any(k in ['url', 'repo', 'path'] for k in args)
                 if dangerous and not self.args['danger']:
-                    raise Exception("""
-                    You are attempting to access a remote file %s.
-                    You must enable --danger flag to do this.""" % glob_string)
+                    msg = "You are attempting to access a remote file %s." % glob_string
+                    msg += " You must specify -danger option to do this.\n"
+                    raise dexy.commands.UserFeedback(msg)
                 glob_string = glob_string.replace("@", "")
             else:
                 virtual = False
@@ -294,8 +299,8 @@ class Controller(object):
 
                 inputs = []
                 if args.has_key('inputs'):
-                    if isinstance(args['inputs'], str):
-                        raise Exception("inputs for %s should be an array" % f)
+                    if isinstance(args['inputs'], str) or isinstance(args['inputs'], unicode):
+                        raise dexy.commands.UserFeedback("inputs for %s should be an array" % f)
                     for i in args['inputs']:
                         # Create document objects for input patterns (just in this directory)
                         for doc in parse_doc(path, i):
@@ -339,7 +344,7 @@ class Controller(object):
                     try:
                         except_re = re.compile(args['except'])
                     except sre_constants.error as e:
-                        raise Exception("""You passed 'except' value of %s.
+                        raise dexy.commands.UserFeedback("""You passed 'except' value of %s.
 Please pass a valid Python-style regular expression for
 'except', NOT a glob-style matcher. Error message from
 re.compile: %s""" % (args['except'], e))
@@ -370,19 +375,13 @@ re.compile: %s""" % (args['except'], e))
                     self.log.debug("creating doc %s for glob %s" % (key, glob_string))
 
                     if self.members.has_key(key):
-                        self.log.debug("found existing key %s" % key)
                         doc = self.members[key]
-                    else:
-                        self.log.debug("no existing key %s" % key)
 
                     if args.has_key('priority'):
                         doc.priority = args['priority']
                         del args['priority']
 
-                    self.log.debug("Doc args: '%s'" % doc.args)
-                    self.log.debug("updating with: '%s'" % args)
                     doc.args.update(args)
-                    self.log.debug("Doc args after update: '%s'" % doc.args)
 
                     if args.has_key('allinputs'):
                         doc.use_all_inputs = args['allinputs']
@@ -411,7 +410,8 @@ re.compile: %s""" % (args['except'], e))
         self.depends = []
 
         self.batch_id = self.db.next_batch_id()
-        print "batch id is", self.batch_id
+        if not self.args['silent']:
+            print "batch id is", self.batch_id
 
         for path, config in self.config.iteritems():
             ### @export "features-global-args-1"
@@ -447,7 +447,14 @@ re.compile: %s""" % (args['except'], e))
             total_dependencies += len(doc.inputs)
             for input_doc in doc.inputs:
                 depend(doc, input_doc)
-            self.log.debug("finalizing dependencies for %s" % doc.key())
+
+            self.log.debug("finalized dependencies for %s" % doc.key())
+            if len(doc.inputs) > 10:
+                self.log.debug("%s inputs added" % len(doc.inputs))
+            elif len(doc.inputs) == 0:
+                self.log.debug("no inputs added")
+            else:
+                self.log.debug("inputs added: %s" % ", ".join(d.key() for d in doc.inputs))
 
         if len(self.args['run']) > 0:
             # Only run the specified document, and its dependencies.
@@ -470,10 +477,16 @@ re.compile: %s""" % (args['except'], e))
                     parse_new_document(input_doc)
 
             run_key = self.args['run']
-            doc = self.members[run_key]
+            if self.members.has_key(run_key):
+                doc = self.members[run_key]
+            else:
+                matches = [k for k in self.members.keys() if k.startswith(run_key)]
+                matches.sort(key=lambda k: len(self.members[k].inputs))
+                doc = self.members[matches[-1]]
             parse_new_document(doc)
 
-            print "limiting members list to %s and its dependencies, %s/%s documents will be run" % (doc.key(), len(new_members), len(self.members))
+            if not self.args['silent']:
+                print "limiting members list to %s and its dependencies, %s/%s documents will be run" % (doc.key(), len(new_members), len(self.members))
             self.members = new_members
             self.depends = new_depends
 
@@ -483,13 +496,14 @@ re.compile: %s""" % (args['except'], e))
         else:
             dep_ratio = None
 
-        print "sorting %s documents into run order, there are %s total dependencies" % (num_members, total_dependencies)
-        if dep_ratio:
-            print "ratio of dependencies to documents is %0.1f" % (dep_ratio)
-            if dep_ratio > 10:
-                print "if you are experiencing performance problems:"
-                print "call dexy with -dryrun and inspect logs/batch-XXXX.json to debug dependencies"
-                print "consider using -strictinherit or reducing your use of 'allinputs' "
+        if not self.args['silent']:
+            print "sorting %s documents into run order, there are %s total dependencies" % (num_members, total_dependencies)
+            if dep_ratio:
+                print "ratio of dependencies to documents is %0.1f" % (dep_ratio)
+                if dep_ratio > 10:
+                    print "if you are experiencing performance problems:"
+                    print "call dexy with -dryrun and inspect logs/batch-XXXX.json to debug dependencies"
+                    print "consider using -strictinherit or reducing your use of 'allinputs' "
 
         try:
             self.log.debug("Beginning topological sort...")
@@ -501,7 +515,7 @@ re.compile: %s""" % (args['except'], e))
             for child, parents in children.items():
                 for parent in parents:
                     print "%s depends on %s" % (self.members.keys()[parent], self.members.keys()[child])
-            raise e
+            raise dexy.commands.UserFeedback(e.message)
 
         docs_without_dependencies = frozenset(range(len(self.members))) - frozenset(topsort_ordering)
         self.ordering = topsort_ordering + list(docs_without_dependencies)
