@@ -1,6 +1,7 @@
 from dexy.constants import Constants
 import StringIO
 import codecs
+import dexy.commands
 import dexy.controller
 import dexy.introspect
 import fnmatch
@@ -55,7 +56,7 @@ class Document(object):
         self.batch_id = controller.batch_id
 
     def setup_log(self):
-        self.log = logging.getLogger(self.key())
+        self.log = logging.getLogger("%s (batch %s)" % (self.key(), self.batch_id))
 
         if hasattr(self, 'loglevelname'):
             loglevelname = self.loglevelname
@@ -84,7 +85,7 @@ class Document(object):
 
     def get_filter_for_alias(self, alias):
         if alias == '':
-            raise Exception("You have a trailing | or 2 | symbols together in your specification for %s" % self.key())
+            raise dexy.commands.UserFeedback("You have a trailing | or 2 | symbols together in your specification for %s" % self.key())
         else:
             return dexy.introspect.get_filter_for_alias(alias, self.__class__.filter_list)
 
@@ -104,7 +105,7 @@ class Document(object):
         will depend on (have as inputs).
         """
         start = time.time()
-        if len(self.input_args) > 0 or len(self.input_keys) > 0 or self.use_all_inputs:
+        if len(self.input_args) > 0 or len(self.input_keys) > 0 or self.use_all_inputs or self.args.has_key('exact-inputs'):
             for doc in members_dict.values():
                 if doc.key() in self.input_keys:
                     self.inputs.append(doc)
@@ -185,7 +186,7 @@ class Document(object):
             k = i.key()
             a = i.final_artifact()
             if not a:
-                raise Exception("No final artifact for document %s" % i.key())
+                raise dexy.commands.InternalDexyProblem("No final artifact for document %s" % i.key())
             input_artifacts[k] = a
         return input_artifacts
 
@@ -279,7 +280,7 @@ class Document(object):
 
         elif self.virtual and (self.args.has_key('repo') and self.args.has_key('path')):
             if not USE_GIT:
-                raise Exception("you can't use repo/path unless you install GitPython")
+                raise dexy.commands.UserFeedback("you can't use repo/path unless you install GitPython")
             repo_url = self.args['repo']
             digest = hashlib.md5(repo_url).hexdigest()
             local_repo_dir = os.path.join(self.artifacts_dir, "repository-%s" % digest)
@@ -290,13 +291,17 @@ class Document(object):
                 assert o.url == repo_url, "local repo exists but url %s does not match requested url %s" % (o.url, repo_url)
                 o.pull() # TODO remember last pulled time so that don't do this too often? what if network unavailable?
             else:
-                repo = git.Repo.clone_from(repo_url, local_repo_dir)
-
+                if self.args.has_key('branch'):
+                    branch = self.args['branch']
+                    repo = git.Repo.clone_from(repo_url, local_repo_dir, branch=branch)
+                else:
+                    repo = git.Repo.clone_from(repo_url, local_repo_dir)
+            
             if self.args.has_key('commit'):
                 commit = self.args['commit']
                 tree = repo.commit(commit).tree
             else:
-                tree = repo.heads.master.commit.tree
+                tree = repo.heads[0].commit.tree
 
             try:
                 blob = tree[self.args['path']]
@@ -319,7 +324,6 @@ class Document(object):
     def create_initial_artifact(self):
         artifact = self.artifact_class.setup(self, self.name)
         if len(self.filters) == 0:
-            artifact.is_last = True
             if artifact.final is None:
                 artifact.final = True
         artifact.save()
@@ -328,7 +332,9 @@ class Document(object):
                 if not os.path.exists(artifact.filepath()):
                     # make a fake file
                     with open(artifact.filepath(), "wb") as f:
-                        f.write("virtual file")
+                        initial_artifact_data = artifact.doc.initial_artifact_data()
+                        if initial_artifact_data:
+                            f.write(artifact.doc.initial_artifact_data())
             else:
                 shutil.copyfile(artifact.name, artifact.filepath())
         return artifact
@@ -360,6 +366,7 @@ class Document(object):
 
             FilterClass = self.get_filter_for_alias(f)
             artifact = self.artifact_class.setup(self, artifact_key, FilterClass, previous_artifact)
+            artifact.filter_alias = f
             self.timing.append(("setup-step-%s" % self.step, time.time() - start))
             start=time.time()
 
@@ -375,7 +382,6 @@ class Document(object):
             self.timing.append(("append-step-%s" % self.step, time.time() - start))
             start = time.time()
 
-        self.last_artifact.is_last = True
         self.last_artifact.save_meta()
         self.db.append_artifacts(self.artifacts)
         self.timing.append(("append-artifacts", time.time() - start))
@@ -396,7 +402,7 @@ class Document(object):
                 except IOError:
                     # no file was created, something went wrong in the script that should have
                     # generated this artifact
-                    raise Exception("""You requested an additional file named %s
+                    raise dexy.commands.UserFeedback("""You requested an additional file named %s
                     but its artifact file (%s) does not exist. Something may have gone wrong
                     in one of the filters of %s""" % (a.key, a.filepath(), a.created_by))
 
